﻿namespace iFileProxy.Middleware
{
    using iFileProxy.Config;
    using iFileProxy.Helpers;
    using iFileProxy.Models;
    using Microsoft.AspNetCore.Http;
    using System;
    using System.Collections.Generic;
    using System.Text.Json;
    using System.Threading.Tasks;

    public class IPAccessLimitMiddleware(RequestDelegate next, Dictionary<string, Dictionary<string, uint>> IPAccessCountDict, AppConfig appConfig)
    {
        private readonly RequestDelegate _next = next;
        private readonly Dictionary<string, Dictionary<string, uint>> _IPAccessCountDict = IPAccessCountDict;

        public async Task InvokeAsync(HttpContext context)
        {
            if (appConfig.SecurityOptions.BlockedClientIP.IndexOf(MasterHelper.GetClientIPAddr(context)) != -1)
            {
                context.Response.StatusCode = 403;
                await context.Response.WriteAsJsonAsync(new CommonRsp { Retcode = 403, Message = "你的IP地址已经被管理员拉入黑名单!"});
                return;
            }

            // 获取需要跟踪的路由列表
            var routesToTrack = appConfig.SecurityOptions.RoutesToTrack;

            // 检查当前请求的路径是否在需要跟踪的路由列表中
            foreach (var p in routesToTrack)
            {
                if (context.Request.Path.ToString().StartsWith(p, StringComparison.OrdinalIgnoreCase))
                {
                    // 如果匹配到需要跟踪的路由，继续处理请求
                    break;
                }
            }

            // 如果没有匹配到需要跟踪的路由，直接调用下一个中间件
            if (!routesToTrack.Any(p => context.Request.Path.ToString().StartsWith(p, StringComparison.OrdinalIgnoreCase)))
            {
                await _next(context);
                return;
            }

            string dateStr = DateTime.Now.ToString("yyyy-MM-dd");
            string ipStr = MasterHelper.GetClientIPAddr(context);
            
            if (_IPAccessCountDict.ContainsKey(dateStr))
            {
                if (_IPAccessCountDict[dateStr].ContainsKey(ipStr))
                {
                    if (_IPAccessCountDict[dateStr][ipStr] >= appConfig.SecurityOptions.DailyRequestLimitPerIP)
                    {
                        context.Response.StatusCode = 200;
                        context.Response.ContentType = "application/json";
                        await context.Response.WriteAsync(JsonSerializer.Serialize(new CommonRsp { Retcode = 403, Message = "日请求次数超过限制! (GMT+8 时间00:00重置)" }));
                        return;
                    }
                    _IPAccessCountDict[dateStr][ipStr]++;
                }
                else
                {
                    _IPAccessCountDict[dateStr].Add(ipStr, 1);
                }
            }
            else
            {
                _IPAccessCountDict.Add(dateStr, new Dictionary<string, uint> { { ipStr, 1 } });
            }

            await _next(context);
        }
    }
}
